package com.example.arduino.phone;

import android.Manifest;
import android.bluetooth.BluetoothSocket;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.Handler;
import android.os.Message;
import android.provider.ContactsContract;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.widget.Button;
import android.widget.TextView;
import android.view.View;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

public class MainActivity extends AppCompatActivity {
    private TextView tvNumbers;
    private TextView tvName;
    private TextView tvStatus;
    private TextView tvLog;
    private BluetoothAdapter baBlue;
    private String sAddress = "98:D3:31:FB:18:54";
    private UUID bUUID = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");
    private BluetoothSocket bsSock = null;
    private Thread mRun = null;
    private byte[] baDigits = new byte[1024];
    private int iCount = 0;
    private Map<String, Integer> mHistory = new HashMap<String, Integer>();
    private boolean bExist = false;

    final int BT_READ = 0;
    final int BT_ADAPTER = 1;
    final int BT_SOCK = 2;
    final int BT_CONNECT = 3;
    final int BT_RFAIL = 4;
    final int BT_CLOSE = 5;
    final int BT_READY = 6;
    private Handler mHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case BT_READ:
                    boolean bDone = false;
                    byte[] buffer = (byte[])msg.obj;
                    for (int i = 0; i < msg.arg1; i++) {
                        if (buffer[i] == 114) {
                            iCount = 0;
                        }
                        else if (buffer[i] == 115) {
                            bDone = true;
                            break;
                        }
                        else {
                            baDigits[iCount] = buffer[i];
                            iCount++;
                        }
                    }
                    if (bDone && iCount > 0) {
                        tvStatus.setText(R.string.searching);
                        String s = new String(baDigits, 0, iCount);
                        searchContacts(s);
                    }
                    break;
                case BT_ADAPTER:
                    tvStatus.setText("No bluetooth adapter found");
                    break;
                case BT_SOCK:
                    tvStatus.setText("cannot create socket");
                    break;
                case BT_CONNECT:
                    tvStatus.setText(R.string.connect);
                    break;
                case BT_RFAIL:
                    tvStatus.setText("cannot read from bluetooth");
                    break;
                case BT_CLOSE:
                    tvStatus.setText("bluetooth closed");
                    break;
                case BT_READY:
                    tvStatus.setText(R.string.ready);
                    break;
            }
        }
    };

    private void readSavedHistory() {
        bExist = true;
        try {
            FileInputStream fIn = openFileInput("history.txt");
            InputStreamReader isr = new InputStreamReader(fIn);
            BufferedReader buffreader = new BufferedReader(isr);
            String line = buffreader.readLine();
            while (line != null) {
                String[] parts = line.split(" ");
                mHistory.put(parts[0], Integer.parseInt(parts[1]));
                line = buffreader.readLine();
            }
            isr.close();
        }
        catch (FileNotFoundException e) {
            bExist = false;
        }
        catch (IOException e) {
            bExist = false;
        }
    }

    private void writeHistory() {
        try {
            FileOutputStream fOut = openFileOutput("history.txt", MODE_PRIVATE);
            OutputStreamWriter osw = new OutputStreamWriter(fOut);
            for (String key: mHistory.keySet()) {
                osw.write(key+" "+mHistory.get(key)+"\n");
            }
            osw.close();
        }
        catch (IOException e) {
            tvStatus.setText("history.txt is NOT saved");
        }
    }

    private String readLog() {
        String log = "";
        try {
            FileInputStream fIn = openFileInput("log.txt");
            InputStreamReader isr = new InputStreamReader(fIn);
            BufferedReader buffreader = new BufferedReader(isr);
            String line = buffreader.readLine();
            while (line != null) {
                log += line;
                line = buffreader.readLine();
            }
            isr.close();
        }
        catch (FileNotFoundException e) {
            log = "No log.txt";
        }
        catch (IOException e) {
            log = "Can not read log.txt";
        }
        return log;
    }

    private void writeLog(String s) {
        try {
            FileOutputStream fOut = openFileOutput("log.txt", MODE_APPEND);
            OutputStreamWriter osw = new OutputStreamWriter(fOut);
            osw.write(s);
            osw.close();
        }
        catch (IOException e) {
            tvStatus.setText("log.txt is NOT saved");
        }
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        tvNumbers = (TextView)findViewById(R.id.numbers);
        tvName = (TextView)findViewById(R.id.name);
        tvStatus = (TextView)findViewById(R.id.status);
        tvLog = (TextView)findViewById(R.id.log);
        readSavedHistory();
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (bsSock != null)
            return;
        mRun = new Thread(new Runnable() {
            @Override
            public void run() {
                baBlue = BluetoothAdapter.getDefaultAdapter();
                InputStream btIn = null;
                if (baBlue != null) {
                    if (baBlue.isEnabled() && bsSock == null) {
                        byte[] buffer = new byte[1024];
                        try {
                            BluetoothDevice bdBlue = baBlue.getRemoteDevice(sAddress);
                            bsSock = bdBlue.createInsecureRfcommSocketToServiceRecord(bUUID);
                            baBlue.cancelDiscovery();
                            try {
                                bsSock.connect();
                                btIn = bsSock.getInputStream();
                            }
                            catch (IOException e) {
                                Message msg = mHandler.obtainMessage(BT_CONNECT, 0, 0, null);
                                msg.sendToTarget();
                                try {
                                    bsSock.close();
                                }
                                catch (IOException e2) {
                                    msg = mHandler.obtainMessage(BT_CLOSE, 0, 0, null);
                                    msg.sendToTarget();
                                }
                                bsSock = null;
                            }
                        }
                        catch (IOException e) {
                            Message msg = mHandler.obtainMessage(BT_SOCK, 0, 0, null);
                            msg.sendToTarget();
                            bsSock = null;
                        }
                        if (bsSock == null)
                            return;
                        try {
                            Message msg = mHandler.obtainMessage(BT_READY, 0, 0, null);
                            msg.sendToTarget();
                            while (!Thread.interrupted()) {
                                if (btIn.available() > 0) {
                                    final int count = btIn.read(buffer, 0, 1024);
                                    msg = mHandler.obtainMessage(BT_READ, count, -1, buffer);
                                    msg.sendToTarget();
                                }
                            }
                        }
                        catch (IOException e) {
                            Message msg = mHandler.obtainMessage();
                            msg.sendToTarget();
                            try {
                                bsSock.close();
                            }
                            catch (IOException e2) {
                                msg = mHandler.obtainMessage(BT_CLOSE, 0, 0, null);
                                msg.sendToTarget();
                            }
                            bsSock = null;
                        }
                    }
                    else {
                        Intent turnOn = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
                        startActivityForResult(turnOn, 1);
                    }
                }
                else {
                    Message msg = mHandler.obtainMessage(BT_ADAPTER, 0, 0, null);
                    msg.sendToTarget();
                }
            }
        });
        mRun.start();
    }

    @Override
    protected void onPause() {
        super.onPause();
        if (bsSock != null) {
            if (mRun != null) {
                try {
                    mRun.interrupt();
                }
                catch (SecurityException e) {
                    tvStatus.setText("cannont interrupt");
                }
            }
            try {
                bsSock.close();
            }
            catch (IOException e) {
                tvStatus.setText("cannot close bluetooth socket");
            }
            bsSock = null;
        }
    }

    private  int getDistance(String num1, String num2) {
        int[][] dp = new int[num1.length()+1][num2.length()+1];
        for (int i = 0; i <= num1.length(); i++) {
            for (int j = 0; j <= num2.length(); j++) {
                if (i == 0) {
                    dp[i][j] = j;
                }
                else if (j == 0) {
                    dp[i][j] = i;
                }
                else {
                    int c = 0;
                    if (num1.charAt(i-1) != num2.charAt(j-1))
                        c = 1;
                    dp[i][j] = Math.min(Math.min(dp[i-1][j-1]+c, dp[i-1][j]+1), dp[i][j-1]);
                }
            }
        }
        return dp[num1.length()][num2.length()];
    }

    private void searchContacts(String digits) {
        String[] projection = new String[] {
                ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME,
                ContactsContract.CommonDataKinds.Phone.NUMBER
        };
        Uri contactUri = ContactsContract.CommonDataKinds.Phone.CONTENT_URI;
        Cursor cursor = getContentResolver().query(contactUri, projection, null, null, null);
        String bestName;
        String bestNum;
        int bestDist;
        int bestCnt;
        if (cursor != null && cursor.moveToFirst()) {
            String num = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER));
            int dist = getDistance(digits, num);
            int cnt = mHistory.getOrDefault(num, 0);
            bestCnt = cnt;
            bestDist = dist;
            bestNum = num;
            bestName = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME));
            while(cursor.moveToNext()) {
                num = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.NUMBER));
                dist = getDistance(digits, num);
                cnt = mHistory.getOrDefault(num, 0);
                if ((dist < bestDist) || ((dist == bestDist) && (cnt > bestCnt))) {
                    bestCnt = cnt;
                    bestDist = dist;
                    bestNum = num;
                    bestName = cursor.getString(cursor.getColumnIndex(ContactsContract.CommonDataKinds.Phone.DISPLAY_NAME));
                }
            }
            if (bestDist <= 5) {
                mHistory.putIfAbsent(bestNum, bestCnt);
                bestCnt ++;
                mHistory.replace(bestNum, bestCnt);
                Calendar c = Calendar.getInstance();
                SimpleDateFormat dateFormat = new SimpleDateFormat(" dd-MM-yyyy HH:mm/");
                String datetime = dateFormat.format(c.getTime());
                writeLog(datetime+digits+"/"+bestNum+"\n");
                tvNumbers.setText(bestNum);
                tvName.setText(bestName);
                tvStatus.setText(R.string.bestdist);
                String number = tvNumbers.getText().toString();
                int pc = ContextCompat.checkSelfPermission(this, Manifest.permission.CALL_PHONE);
//                number = "0970083429";
                Intent intent = new Intent(Intent.ACTION_CALL, Uri.parse("tel:"+number));
//                intent.setClassName("com.android.phone","com.android.phone.OutgoingCallBroadcaster");
                intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
                startActivity(intent);
                cursor.close();
                finish();
            }
            else {
                tvNumbers.setText(digits);
                tvName.setText("");
                tvStatus.setText(R.string.distance);
                cursor.close();
            }
        }
        else
            tvStatus.setText("Nothing found");
    }

    public void doShowLog(View view) {
        String log = readLog();
        tvLog.setText(log);
    }

    public void doDial(View view) {
        String number = tvNumbers.getText().toString();
        if (number.compareTo(getString(R.string.numbers)) == 0) {
            tvStatus.setText(R.string.no_number);
            return;
        }
        int pc = ContextCompat.checkSelfPermission(this, Manifest.permission.CALL_PHONE);
        Intent intent = new Intent(Intent.ACTION_CALL, Uri.parse("tel:"+number));
//        intent.setClassName("com.android.phone","com.android.phone.OutgoingCallBroadcaster");
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
        startActivity(intent);
    }

    public void doEnd(View view) {
        writeHistory();
        finish();
    }
}
